@@ -5,22 +5,19 @@ import os
 import re
 import shutil
 import sys
-import tempfile
-from subprocess import Popen
 
 import jinja2
 from six import binary_type
 from six import string_types
 from six.moves import configparser
 
-from . import macro as macro_module
 from . import __version__
+from . import macro as macro_module
 from . import utils
 from .parser import Parser
 
 BASE_DIR = os.path.dirname(__file__)
 THEMES_DIR = os.path.join(BASE_DIR, 'themes')
-TOC_MAX_LEVEL = 2
 VALID_LINENOS = ('no', 'inline', 'table')
 
 
@@ -45,13 +42,14 @@ class Generator(object):
             Available ``kwargs`` are:
             - ``copy_theme``: copy theme directory and files into presentation
                               one
-            - ``destination_file``: path to html or PDF destination file
+            - ``destination_file``: path to html destination file
             - ``direct``: enables direct rendering presentation to stdout
             - ``debug``: enables debug mode
             - ``embed``: generates a standalone document, with embedded assets
             - ``encoding``: the encoding to use for this presentation
             - ``extensions``: Comma separated list of markdown extensions
             - ``logger``: a logger lambda to use for logging
+            - ``maxtoclevel``: the maximum level to include in toc
             - ``presenter_notes``: enable presenter notes
             - ``relative``: enable relative asset urls
             - ``theme``: path to the theme to use for this presentation
@@ -68,6 +66,7 @@ class Generator(object):
         self.encoding = kwargs.get('encoding', 'utf8')
         self.extensions = kwargs.get('extensions', None)
         self.logger = kwargs.get('logger', None)
+        self.maxtoclevel = kwargs.get('maxtoclevel', 2)
         self.presenter_notes = kwargs.get('presenter_notes', True)
         self.relative = kwargs.get('relative', False)
         self.theme = kwargs.get('theme', 'default')
@@ -96,10 +95,11 @@ class Generator(object):
             self.relative = config.get('relative', self.relative)
             self.copy_theme = config.get('copy_theme', self.copy_theme)
             self.extensions = config.get('extensions', self.extensions)
+            self.maxtoclevel = config.get('max-toc-level', self.maxtoclevel)
             self.theme = config.get('theme', self.theme)
             self.destination_dir = os.path.dirname(self.destination_file)
-            self.add_user_css(config.get('css', []))
-            self.add_user_js(config.get('js', []))
+            self.user_css.extend(self.process_user_files(config.get('css', [])))
+            self.user_js.extend(self.process_user_files(config.get('js', [])))
             self.linenos = self.linenos_check(config.get('linenos', self.linenos))
         else:
             self.source = source
@@ -116,16 +116,6 @@ class Generator(object):
         if os.path.exists(self.destination_file) and not os.path.isfile(self.destination_file):
             raise IOError("Destination %s exists and is not a file" % self.destination_file)
 
-        if self.destination_file.endswith('.html'):
-            self.file_type = 'html'
-        elif self.destination_file.endswith('.pdf'):
-            self.file_type = 'pdf'
-            self.embed = True
-        else:
-            raise IOError("This program can only write html or pdf files. "
-                          "Please use one of these file extensions in the "
-                          "destination")
-
         self.theme_dir = self.find_theme_dir(self.theme, self.copy_theme)
         self.template_file = self.get_template_file()
 
@@ -133,47 +123,30 @@ class Generator(object):
         self.macros = []
         self.register_macro(*self.default_macros)
 
-    def add_user_css(self, css_list):
-        """ Adds supplementary user css files to the presentation. The
-            ``css_list`` arg can be either a ``list`` or a string.
-        """
-        if isinstance(css_list, string_types):
-            css_list = [css_list]
-
-        for css_path in css_list:
-            if css_path and css_path not in self.user_css:
-                if not os.path.exists(css_path):
-                    raise IOError('%s user css file not found' % (css_path,))
-                with codecs.open(css_path, encoding=self.encoding) as css_file:
-                    self.user_css.append({
-                        'path_url': utils.get_path_url(css_path,
-                                                       self.relative and self.destination_dir),
-                        'contents': css_file.read(),
-                    })
-
-    def add_user_js(self, js_list):
-        """ Adds supplementary user javascript files to the presentation. The
-            ``js_list`` arg can be either a ``list`` or a string.
-        """
-        if isinstance(js_list, string_types):
-            js_list = [js_list]
-        for js_path in js_list:
-            if js_path and js_path not in self.user_js:
-                if js_path.startswith("http:"):
-                    self.user_js.append({
-                        'path_url': js_path,
-                        'contents': '',
-                    })
-                elif not os.path.exists(js_path):
-                    raise IOError('%s user js file not found' % (js_path,))
-                else:
-                    with codecs.open(js_path,
-                                     encoding=self.encoding) as js_file:
-                        self.user_js.append({
-                            'path_url': utils.get_path_url(js_path,
-                                                           self.relative and self.destination_dir),
-                            'contents': js_file.read(),
-                        })
+    def process_user_files(self, files):
+        if isinstance(files, string_types):
+            files = [files]
+        for path in files:
+            if path.startswith(("http://", "https://")):
+                yield {
+                    'path_url': path,
+                    'contents': '',
+                    'dirname': '',
+                    'embeddable': False,
+                }
+                self.log("Loaded:  %s (not embeddable)\n" % path)
+            else:
+                path = os.path.normpath(os.path.join(self.work_dir, path))
+                if not os.path.exists(path):
+                    raise IOError('%s user file not found' % (path,))
+                with codecs.open(path, encoding=self.encoding) as fh:
+                    yield {
+                        'path_url': utils.get_path_url(path, self.relative and self.destination_dir),
+                        'dirname': os.path.dirname(path) or '.',
+                        'contents': fh.read(),
+                        'embeddable': True,
+                    }
+                    self.log("Loaded:  %s\n" % path)
 
     def add_toc_entry(self, title, level, slide_number):
         """ Adds a new entry to current presentation Table of Contents.
@@ -200,12 +173,8 @@ class Generator(object):
         """ Execute this generator regarding its current configuration.
         """
         if self.direct:
-            if self.file_type == 'pdf':
-                raise IOError("Direct output mode is not available for PDF "
-                              "export")
-            else:
-                out = getattr(sys.stdout, 'buffer', sys.stdout)
-                out.write(self.render().encode(self.encoding))
+            out = getattr(sys.stdout, 'buffer', sys.stdout)
+            out.write(self.render().encode(self.encoding))
         else:
             self.write_and_log()
 
@@ -279,23 +248,24 @@ class Generator(object):
         """ Finds them dir path from its name.
         """
         if os.path.exists(theme):
-            self.theme_dir = theme
+            theme_dir = theme
         elif os.path.exists(os.path.join(THEMES_DIR, theme)):
-            self.theme_dir = os.path.join(THEMES_DIR, theme)
+            theme_dir = os.path.join(THEMES_DIR, theme)
         else:
             raise IOError("Theme %s not found or invalid" % theme)
+
         target_theme_dir = os.path.join(os.getcwd(), 'theme')
         if copy_theme or os.path.exists(target_theme_dir):
             self.log(u'Copying %s theme directory to %s'
                      % (theme, target_theme_dir))
             if not os.path.exists(target_theme_dir):
                 try:
-                    shutil.copytree(self.theme_dir, target_theme_dir)
+                    shutil.copytree(theme_dir, target_theme_dir)
                 except Exception as e:
                     self.log(u"Skipped copy of theme folder: %s" % e)
-                    pass
-            self.theme_dir = target_theme_dir
-        return self.theme_dir
+            theme_dir = target_theme_dir
+
+        return theme_dir
 
     def get_css(self):
         """ Fetches and returns stylesheet file path or contents, for both
@@ -313,6 +283,7 @@ class Generator(object):
             css['base'] = {
                 'path_url': utils.get_path_url(base_css, self.relative and self.destination_dir),
                 'contents': css_file.read(),
+                'embeddable': True
             }
 
         print_css = os.path.join(self.theme_dir, 'css', 'print.css')
@@ -324,6 +295,7 @@ class Generator(object):
             css['print'] = {
                 'path_url': utils.get_path_url(print_css, self.relative and self.destination_dir),
                 'contents': css_file.read(),
+                'embeddable': True
             }
 
         screen_css = os.path.join(self.theme_dir, 'css', 'screen.css')
@@ -335,6 +307,7 @@ class Generator(object):
             css['screen'] = {
                 'path_url': utils.get_path_url(screen_css, self.relative and self.destination_dir),
                 'contents': css_file.read(),
+                'embeddable': True
             }
 
         theme_css = os.path.join(self.theme_dir, 'css', 'theme.css')
@@ -346,6 +319,7 @@ class Generator(object):
             css['theme'] = {
                 'path_url': utils.get_path_url(theme_css, self.relative and self.destination_dir),
                 'contents': css_file.read(),
+                'embeddable': True
             }
 
         return css
@@ -365,6 +339,7 @@ class Generator(object):
             return {
                 'path_url': utils.get_path_url(js_file, self.relative and self.destination_dir),
                 'contents': js_file_obj.read(),
+                'embeddable': True
             }
 
     def get_slide_vars(self, slide_src, source,
@@ -374,7 +349,7 @@ class Generator(object):
         """ Computes a single slide template vars from its html source code.
             Also extracts slide information for the table of contents.
         """
-        presenter_notes = None
+        presenter_notes = ''
 
         find = _presenter_notes_re.search(slide_src)
 
@@ -419,10 +394,13 @@ class Generator(object):
                 classes=slide_classes,
                 header=header,
                 level=level,
-                presenter_notes=presenter_notes,
                 source=source_dict,
                 title=title,
             )
+            context.setdefault('presenter_notes', '')
+            context['presenter_notes'] += presenter_notes
+            if not context['presenter_notes']:
+                context['presenter_notes'] = None
             return context
 
     def get_template_vars(self, slides):
@@ -438,12 +416,9 @@ class Generator(object):
                 continue
             self.num_slides += 1
             slide_number = slide_vars['number'] = self.num_slides
-            if slide_vars['level'] and slide_vars['level'] <= TOC_MAX_LEVEL:
-                self.add_toc_entry(slide_vars['title'], slide_vars['level'],
-                                   slide_number)
-            else:
-                # Put something in the TOC even if it doesn't have a title or level
-                self.add_toc_entry(u"-", 1, slide_number)
+            if slide_vars['level'] and slide_vars['level'] <= self.maxtoclevel:
+                # only show slides that have a title and lever is not too deep
+                self.add_toc_entry(slide_vars['title'], slide_vars['level'], slide_number)
 
         return {'head_title': head_title, 'num_slides': str(self.num_slides),
                 'slides': slides, 'toc': self.toc, 'embed': self.embed,
@@ -485,6 +460,8 @@ class Generator(object):
             config['destination'] = raw_config.get(section_name, 'destination')
         if raw_config.has_option(section_name, 'linenos'):
             config['linenos'] = raw_config.get(section_name, 'linenos')
+        if raw_config.has_option(section_name, 'max-toc-level'):
+            config['max-toc-level'] = int(raw_config.get(section_name, 'max-toc-level'))
         for boolopt in ('embed', 'relative', 'copy_theme'):
             if raw_config.has_option(section_name, boolopt):
                 config[boolopt] = raw_config.getboolean(section_name, boolopt)
@@ -517,6 +494,41 @@ class Generator(object):
                 raise TypeError("Couldn't register macro; a macro must inherit"
                                 " from macro.Macro")
 
+    def embed_url_data(self, context, html):
+        """Find all image and fonts referenced in CSS with an ``url()`` function
+        and embed them in base64. Images from the user (i.e. included in its
+        source code, *not* in its CSS) are embedded by the macro
+        `EmbedImagesMacro`.
+        """
+        all_urls = re.findall(r'url\([\"\']?(.*?)[\"\']?\)', html,
+                              re.DOTALL | re.UNICODE)
+        embed_exts = ('.jpg', '.jpeg', '.png', '.gif', '.svg', '.woff2',
+                      '.woff')
+        embed_urls = (url for url in all_urls if url.endswith(embed_exts))
+
+        css_dirs = (
+            [os.path.join(self.theme_dir, 'css')] +
+            [css_entry['dirname'] for css_entry in context['user_css']]
+        )
+
+        for embed_url in embed_urls:
+            embed_url = embed_url.replace('"', '').replace("'", '')
+
+            directory, encoded_url = None, None
+            for directory in css_dirs:
+                encoded_url = utils.encode_data_from_url(embed_url, directory)
+                if encoded_url:
+                    break
+
+            if encoded_url:
+                html = html.replace(embed_url, encoded_url, 1)
+                self.log("Embedded theme file %s from directory %s"
+                         % (embed_url, directory))
+            else:
+                self.log(u"Failed to embed theme file %s" % embed_url)
+
+        return html
+
     def render(self):
         """ Returns generated html code.
         """
@@ -528,37 +540,7 @@ class Generator(object):
         html = template.render(context)
 
         if self.embed:
-            images = re.findall(r'url\(["\']?(.*?\.(?:jpe?g|gif|png|svg)[\'"]?)\)', html, re.DOTALL | re.UNICODE)
-
-            for img_url in images:
-                img_url = img_url.replace('"', '').replace("'", '')
-                if self.theme_dir:
-                    source = os.path.join(self.theme_dir, 'css')
-                else:
-                    source = os.path.join(THEMES_DIR, self.theme, 'css')
-
-                encoded_url = utils.encode_image_from_url(img_url, source)
-                if encoded_url:
-                    html = html.replace(img_url, encoded_url, 1)
-                    self.log("Embedded theme image %s from theme directory %s" % (img_url, source))
-                else:
-                    # Missing file in theme directory. Try user_css folders
-                    found = False
-                    for css_entry in context['user_css']:
-                        directory = os.path.dirname(css_entry['path_url'])
-                        if not directory:
-                            directory = "."
-
-                        encoded_url = utils.encode_image_from_url(img_url, directory)
-
-                        if encoded_url:
-                            found = True
-                            html = html.replace(img_url, encoded_url, 1)
-                            self.log("Embedded theme image %s from directory %s" % (img_url, directory))
-
-                    if not found:
-                        # Missing image file, etc...
-                        self.log(u"Failed to embed theme image %s" % img_url)
+            html = self.embed_url_data(context, html)
 
         return html
 
@@ -566,36 +548,9 @@ class Generator(object):
         """ Writes generated presentation code into the destination file.
         """
         html = self.render()
-
-        if self.file_type == 'pdf':
-            self.write_pdf(html)
-        else:
-            dirname = os.path.dirname(self.destination_file)
-            if dirname and not os.path.exists(dirname):
-                os.makedirs(dirname)
-            with codecs.open(self.destination_file, 'w',
-                             encoding='utf_8') as outfile:
-                outfile.write(html)
-
-    def write_pdf(self, html):
-        """ Tries to write a PDF export from the command line using PrinceXML
-            if available.
-        """
-        try:
-            f = tempfile.NamedTemporaryFile(delete=False, suffix='.html')
-            f.write(html.encode('utf_8', 'xmlcharrefreplace'))
-            f.close()
-        except Exception:
-            raise IOError("Unable to create temporary file, aborting")
-
-        dummy_fh = open(os.path.devnull, 'w')
-
-        try:
-            command = ["prince", f.name, self.destination_file]
-
-            Popen(command, stderr=dummy_fh).communicate()
-        except Exception:
-            raise EnvironmentError("Unable to generate PDF file using "
-                                   "prince. Is it installed and available?")
-        finally:
-            dummy_fh.close()
+        dirname = os.path.dirname(self.destination_file)
+        if dirname and not os.path.exists(dirname):
+            os.makedirs(dirname)
+        with codecs.open(self.destination_file, 'w',
+                         encoding='utf_8') as outfile:
+            outfile.write(html)
