@@ -43,7 +43,7 @@ class CodeHighlightingMacro(Macro):
         r'(<pre.+?>(<code>)?\s?!(\S+?)\n(.*?)(</code>)?</pre>)',
         re.UNICODE | re.MULTILINE | re.DOTALL)
 
-    html_entity_re = re.compile('&(\w+?);')
+    html_entity_re = re.compile(r'&(\w+?);')
 
     def descape(self, string, defs=None):
         """Decodes html entities from a given string"""
@@ -101,7 +101,7 @@ class EmbedImagesMacro(Macro):
         source_dir = os.path.dirname(source)
 
         for image_url, data_url in images:
-            encoded_url = utils.encode_image_from_url(image_url or data_url, source_dir)
+            encoded_url = utils.encode_data_from_url(image_url or data_url, source_dir)
 
             if not encoded_url:
                 self.logger(u"Failed to embed image \"%s\"" % image_url, 'warning')
@@ -170,14 +170,17 @@ class NotesMacro(Macro):
     macro_re = re.compile(r'<p>\.notes:\s?(.*?)</p>')
 
     def process(self, content, source=None, context=None):
-        classes = []
+        notes = []
 
-        new_content = self.macro_re.sub(r'<p class="notes">\1</p>', content)
+        def repl(match):
+            notes.append("<p>%s</p>" % match.group(1))
+            return ''
+        new_content = self.macro_re.sub(repl, content)
 
-        if content != new_content:
-            classes.append(u'has_notes')
+        if context is not None:
+            context['presenter_notes'] = ''.join(notes)
 
-        return new_content, classes
+        return new_content, []
 
 
 class QRMacro(Macro):
