@@ -3,6 +3,9 @@ import base64
 import mimetypes
 import os
 
+# add woff2 font type: not here by default...
+mimetypes.add_type('font/woff2', '.woff2')
+
 
 def get_path_url(path, relative=False):
     """ Returns an absolute or relative path url given a path
@@ -13,7 +16,7 @@ def get_path_url(path, relative=False):
         return os.path.relpath(path, relative)
 
 
-def encode_image_from_url(url, source_path):
+def encode_data_from_url(url, source_path):
     if not url or url.startswith('data:') or url.startswith('file://'):
         return False
 
@@ -23,15 +26,11 @@ def encode_image_from_url(url, source_path):
     real_path = url if os.path.isabs(url) else os.path.join(source_path, url)
 
     if not os.path.exists(real_path):
-        print('%s was not found, skipping' % url)
-
         return False
 
     mime_type, encoding = mimetypes.guess_type(real_path)
 
     if not mime_type:
-        print('Unrecognized mime type for %s, skipping' % url)
-
         return False
 
     try:
