@@ -31,8 +31,8 @@ def test_generator__init__():
 def test_add_user_assets():
     base_dir = os.path.join(DATA_DIR, 'test.md')
     g = Generator(base_dir, logger=logtest)
-    g.add_user_css(os.path.join(DATA_DIR, 'test.css'))
-    g.add_user_js(os.path.join(DATA_DIR, 'test.js'))
+    g.user_css.extend(g.process_user_files(os.path.join(DATA_DIR, 'test.css')))
+    g.user_js.extend(g.process_user_files(os.path.join(DATA_DIR, 'test.js')))
     assert g.user_css[0]['contents'] == '* {color: red;}'
     assert g.user_js[0]['contents'] == "alert('foo');"
 
@@ -121,9 +121,11 @@ def test_get_template_vars():
 def test_process_macros():
     g = Generator(os.path.join(DATA_DIR, 'test.md'))
     # Notes
-    r = g.process_macros('<p>foo</p>\n<p>.notes: bar</p>\n<p>baz</p>', '', {})
-    assert r[0].find('<p class="notes">bar</p>') == 11
-    assert r[1] == [u'has_notes']
+    ctx = {}
+    r = g.process_macros('<p>foo</p>\n<p>.notes: bar</p>\n<p>baz</p>', '', ctx)
+    assert r[0].find('bar') == -1
+    assert r[1] == []
+    assert ctx == {'presenter_notes': '<p>bar</p>'}
     # FXs
     content = '<p>foo</p>\n<p>.fx: blah blob</p>\n<p>baz</p>'
     r = g.process_macros(content, '', {})
@@ -265,8 +267,8 @@ def test_fx_macro_process():
 def test_notes_macro_process():
     m = macro.NotesMacro(logtest)
     r = m.process('<p>foo</p>\n<p>.notes: bar</p>\n<p>baz</p>')
-    assert r[0].find('<p class="notes">bar</p>') == 11
-    assert r[1] == [u'has_notes']
+    assert r[0].find('bar') == -1
+    assert r[1] == []
 
 
 def test_parser__init__():
